/*
	UART0-Treiber
	Andreas Kasper 0525365
	TU-Wien E535
	Embedded Systems Engineering
	
	
	-DUART_STANDALONE beim kompilieren angeben oder nachstehendes '#define UART_STANDALONE' aktivieren, damit man den Treiber ohne weitere Software testen kann.
*/
#define UART_STANDALONE

#include "uart.h"

uint8_t uart_send( uint8_t zeichen );

// Handler fr die UART Receive Funktion
void (*uart_recv_callback)(uint8_t);


SIGNAL( SIG_UART0_RECV )
{
	uart_recv_callback( UDR0 );
}


// Funktion zum testen des UART-Treibers ohne weitere Software
#ifdef UART_STANDALONE

void dummycallback( uint8_t dummy )
{
	uint8_t dummy1;

	dummy1 = uart_send( dummy );

}

#endif


//! Funktion zum Senden eines Zeichen
/**
	Sendet ein Zeichen ber den UART. Diese funktion sendet nur wenn der UART bereit ist, andernfalls wird ein Fehler zurckgegeben
	\param zeichen Gibt das Zeichen, das ber den UART uebertragen werden soll an.
	\return Wenn ein Zeichen erfolgreich uebertragen wird wird UART_OK zurueckgegeben. Bei einem Fehler wird UART_NOT_READY zurueckgegeben
	\sa configure_uart
**/
uint8_t uart_send( uint8_t zeichen )
{
	if ( ( UCSR0A & ( 1 << UDRE0 ) ) ) //UART BEREIT?
	{
		UDR0 = zeichen; // Zeihen senden
		
		return UART_OK;
	}
	else
		return UART_NOT_READY; //Fehler zuruecksenden
}


//! Funktion zum Konfigurieren des UART
/**
	Konriguriert den UART
	\param mode Gibt den Modus in dem der UART initialisiert werden soll an. (UART_8N1, UART_8E1, UART_8O1, UART_8N2, UART_8E2, UART_8O2)
	\param baud Gibt die BAUD-Rate mit der der UART initialisiert werden soll an. (z.b.: 19200)
	\param receivefunction Callbackfunktion fuer empfangene Zeichen
	\sa uart_send
**/
void configure_uart( uint8_t mode, uint32_t baud, void (*receivefunction)( uint8_t ) )
{
	//Baudrate berechnen
	uint16_t baudrate = ( 14745600 / ( 16 * baud ) ) - 1;
	
	//callback-Funktion registrieren
	uart_recv_callback = receivefunction;

	//----- UART initialisieren
		UBRR0H = hi8( baudrate );
		UBRR0L = lo8( baudrate );

		// ASYNC Mode
		clear_bit( UCSR0C, UMSEL0 );

		// STOP BIT SETZEN
		if ( ( mode >= 1 ) && ( mode <= 3 ) )
		{
			// 1 STOP Bit
			clear_bit( UCSR0C, USBS0 );
		}
		else
		{
			// 2 STOP Bit
			set_bit( UCSR0C, USBS0 );
		}

		// PARITY SETZEN
		if ( ( mode == 2 ) || ( mode == 5 ) )
		{
			// EVEN Parity
			set_bit( UCSR0C, UPM01 );
			clear_bit( UCSR0C, UPM00 );
		}
		else if ( ( mode == 1 ) || ( mode == 4 ) )
		{
			// NONE Parity
			clear_bit( UCSR0C, UPM01 );
			clear_bit( UCSR0C, UPM00 );
		}
		else
		{
			// ODD Prity
			set_bit( UCSR0C, UPM01 );
			set_bit( UCSR0C, UPM00 );
		}

		// 8 DATA Bit
		set_bit( UCSR0C, UCSZ00 );
		set_bit( UCSR0C, UCSZ01 );
		clear_bit( UCSR0B, UCSZ02 );


		// TX Enable
		set_bit( UCSR0B, TXEN0 );

		// RX Enable
		set_bit( UCSR0B, RXEN0);

		// RX Interrupt Enable
		set_bit( UCSR0B, RXCIE0 );


}


// Funktion zum testen des UART-Treibers ohne weitere Software
#ifdef UART_STANDALONE

int main()
{
	uint8_t dummy;

	//UART konrigurieren
	configure_uart( UART_8N1, 19200, dummycallback );
	
	//Interrupts aktivieren
	sei();

		do
		{

			dummy = uart_send( 'a' );
		} while ( 1 );


	return 0;
}

#endif
